from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Access")


@_attrs_define
class Access:
    """
    Attributes:
        automobiles (bool | Unset): Applicable to automobiles.

            - `true`: The road allows automobiles.
            - `false`: The road doesn't allow automobiles.
        bicycles (bool | Unset): Applicable to bicycles.

            - `true`: The road allows bicycles.
            - `false`: The road doesn't allow bicycles.
        buses (bool | Unset): Applicable to buses.

            - `true`: The road allows buses.
            - `false`: The road doesn't allow buses.
        carpools (bool | Unset): Applicable to carpools.

            - `true`: The road allows carpooling.
            - `false`: The road doesn't allow carpooling.
        deliveries (bool | Unset): Applicable to deliveries.

            - `true`: The road allows deliveries.
            - `false`: The road doesn't allow deliveries.
        emergency_vehicles (bool | Unset): Applicable to emergency vehicles.

            - `true`: The road allows emergency vehicles.
            - `false`: The road doesn't allow emergency vehicles.
        motorcycles (bool | Unset): Applicable to motorcycles.

            - `true`: The road allows motorcycles.
            - `false`: The road doesn't allow motorcycles.
        pedestrians (bool | Unset): Applicable to pedestrians.

            - `true`: The road allows pedestrians.
            - `false`: The road doesn't allow pedestrians.
        taxis (bool | Unset): Applicable to taxis.

            - `true`: The road allows taxis.
            - `false`: The road doesn't allow taxis.
        through_traffic (bool | Unset): Applicable to through traffic.

            - `true`: The road allows through traffic.
            - `false`: The road doesn't allow through traffic.
        trucks (bool | Unset): Applicable to trucks.

            - `true`: The road allows trucks.
            - `false`: The road doesn't allow trucks.
    """

    automobiles: bool | Unset = UNSET
    bicycles: bool | Unset = UNSET
    buses: bool | Unset = UNSET
    carpools: bool | Unset = UNSET
    deliveries: bool | Unset = UNSET
    emergency_vehicles: bool | Unset = UNSET
    motorcycles: bool | Unset = UNSET
    pedestrians: bool | Unset = UNSET
    taxis: bool | Unset = UNSET
    through_traffic: bool | Unset = UNSET
    trucks: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        automobiles = self.automobiles

        bicycles = self.bicycles

        buses = self.buses

        carpools = self.carpools

        deliveries = self.deliveries

        emergency_vehicles = self.emergency_vehicles

        motorcycles = self.motorcycles

        pedestrians = self.pedestrians

        taxis = self.taxis

        through_traffic = self.through_traffic

        trucks = self.trucks

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if automobiles is not UNSET:
            field_dict["automobiles"] = automobiles
        if bicycles is not UNSET:
            field_dict["bicycles"] = bicycles
        if buses is not UNSET:
            field_dict["buses"] = buses
        if carpools is not UNSET:
            field_dict["carpools"] = carpools
        if deliveries is not UNSET:
            field_dict["deliveries"] = deliveries
        if emergency_vehicles is not UNSET:
            field_dict["emergencyVehicles"] = emergency_vehicles
        if motorcycles is not UNSET:
            field_dict["motorcycles"] = motorcycles
        if pedestrians is not UNSET:
            field_dict["pedestrians"] = pedestrians
        if taxis is not UNSET:
            field_dict["taxis"] = taxis
        if through_traffic is not UNSET:
            field_dict["throughTraffic"] = through_traffic
        if trucks is not UNSET:
            field_dict["trucks"] = trucks

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        automobiles = d.pop("automobiles", UNSET)

        bicycles = d.pop("bicycles", UNSET)

        buses = d.pop("buses", UNSET)

        carpools = d.pop("carpools", UNSET)

        deliveries = d.pop("deliveries", UNSET)

        emergency_vehicles = d.pop("emergencyVehicles", UNSET)

        motorcycles = d.pop("motorcycles", UNSET)

        pedestrians = d.pop("pedestrians", UNSET)

        taxis = d.pop("taxis", UNSET)

        through_traffic = d.pop("throughTraffic", UNSET)

        trucks = d.pop("trucks", UNSET)

        access = cls(
            automobiles=automobiles,
            bicycles=bicycles,
            buses=buses,
            carpools=carpools,
            deliveries=deliveries,
            emergency_vehicles=emergency_vehicles,
            motorcycles=motorcycles,
            pedestrians=pedestrians,
            taxis=taxis,
            through_traffic=through_traffic,
            trucks=trucks,
        )

        access.additional_properties = d
        return access

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.connection import Connection
    from ..models.polygon import Polygon
    from ..models.response_range import ResponseRange


T = TypeVar("T", bound="Isoline")


@_attrs_define
class Isoline:
    """An isoline for the specified range parameter.

    Attributes:
        range_ (ResponseRange): Range specified in terms of distance, travel time or energy consumption.
        polygons (list[Polygon]): A set of multiple polygons.
        connections (list[Connection] | Unset): Connections represent the geometry of special links that were reached
            but not included in the components.
            These links are connections like ferries.
    """

    range_: ResponseRange
    polygons: list[Polygon]
    connections: list[Connection] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        range_ = self.range_.to_dict()

        polygons = []
        for componentsschemas_multi_polygon_item_data in self.polygons:
            componentsschemas_multi_polygon_item = (
                componentsschemas_multi_polygon_item_data.to_dict()
            )
            polygons.append(componentsschemas_multi_polygon_item)

        connections: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.connections, Unset):
            connections = []
            for connections_item_data in self.connections:
                connections_item = connections_item_data.to_dict()
                connections.append(connections_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "range": range_,
            "polygons": polygons,
        })
        if connections is not UNSET:
            field_dict["connections"] = connections

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.connection import Connection
        from ..models.polygon import Polygon
        from ..models.response_range import ResponseRange

        d = dict(src_dict)
        range_ = ResponseRange.from_dict(d.pop("range"))

        polygons = []
        _polygons = d.pop("polygons")
        for componentsschemas_multi_polygon_item_data in _polygons:
            componentsschemas_multi_polygon_item = Polygon.from_dict(
                componentsschemas_multi_polygon_item_data
            )

            polygons.append(componentsschemas_multi_polygon_item)

        _connections = d.pop("connections", UNSET)
        connections: list[Connection] | Unset = UNSET
        if _connections is not UNSET:
            connections = []
            for connections_item_data in _connections:
                connections_item = Connection.from_dict(connections_item_data)

                connections.append(connections_item)

        isoline = cls(
            range_=range_,
            polygons=polygons,
            connections=connections,
        )

        isoline.additional_properties = d
        return isoline

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ev_availability_station import EvAvailabilityStation


T = TypeVar("T", bound="EvAvailabilityAttributes")


@_attrs_define
class EvAvailabilityAttributes:
    """
    Attributes:
        stations (list[EvAvailabilityStation] | Unset): List of EV stations at the place (Currently only one is
            expected)
    """

    stations: list[EvAvailabilityStation] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        stations: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.stations, Unset):
            stations = []
            for stations_item_data in self.stations:
                stations_item = stations_item_data.to_dict()
                stations.append(stations_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if stations is not UNSET:
            field_dict["stations"] = stations

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ev_availability_station import EvAvailabilityStation

        d = dict(src_dict)
        _stations = d.pop("stations", UNSET)
        stations: list[EvAvailabilityStation] | Unset = UNSET
        if _stations is not UNSET:
            stations = []
            for stations_item_data in _stations:
                stations_item = EvAvailabilityStation.from_dict(stations_item_data)

                stations.append(stations_item)

        ev_availability_attributes = cls(
            stations=stations,
        )

        ev_availability_attributes.additional_properties = d
        return ev_availability_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Allow")


@_attrs_define
class Allow:
    """Explicitly allow features that require users to opt in.

    Attributes:
        hov (bool | Unset): This parameter specifies whether HOV lanes can be used in the calculation.

            An HOV (High occupancy Vehicle) lane or carpool lane is reserved for carpool usage.
            Carpool lane requires a minimum number of passengers in order for the car to use the
            carpool lane.

            **Notes**:
              - This parameter can't be used with 'vehicle[hovOccupancy]'.
              - This parameter should be used with `vehicle[occupancy]`. If `vehicle[occupancy]` is set, then only HOV lanes
            allowing this number of occupants will be allowed. If `vehicle[occupancy]` is not set, occupancy requirements
            are always considered fulfilled.
              - In case of violation, `violatedCarpool` notice will be returned.
             Default: False.
        hot (bool | Unset): This parameter specifies whether HOT lanes can be used in the calculation.

            HOT (high-occupancy toll) lanes are HOV lanes where vehicles that do not qualify as high-occupancy are allowed
            to pass by paying a toll.

            **Notes**:
              - This parameter can't be used with 'vehicle[hovOccupancy]'.
              - This parameter can be used with `allow[hov]`.
              - In case of violation, `violatedCarpool` notice will be returned.
              - No toll information is returned for HOT lanes since it is dynamic information.
             Default: False.
    """

    hov: bool | Unset = False
    hot: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        hov = self.hov

        hot = self.hot

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if hov is not UNSET:
            field_dict["hov"] = hov
        if hot is not UNSET:
            field_dict["hot"] = hot

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        hov = d.pop("hov", UNSET)

        hot = d.pop("hot", UNSET)

        allow = cls(
            hov=hov,
            hot=hot,
        )

        allow.additional_properties = d
        return allow

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FunctionalClassValue(str, Enum):
    VALUE_0 = "1"
    VALUE_1 = "2"
    VALUE_2 = "3"
    VALUE_3 = "4"
    VALUE_4 = "5"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.bounding_box_area import BoundingBoxArea
    from ..models.corridor_area import CorridorArea
    from ..models.encoded_corridor_area import EncodedCorridorArea
    from ..models.encoded_polygon_area import EncodedPolygonArea
    from ..models.polygon_area import PolygonArea


T = TypeVar("T", bound="BoundingBoxAreaWithExceptions")


@_attrs_define
class BoundingBoxAreaWithExceptions:
    """A bounding box defined by two longitudes and two latitudes.
    Can be expanded to include excepted areas that are excluded from the current bounding box area.

        Attributes:
            type_ (str):  Example: boundingBox.
            north (float): Latitude in WGS-84 degrees of the northern boundary of the box. Example: 30.0.
            south (float): Latitude in WGS-84 degrees of the southern boundary of the box. Example: 30.0.
            east (float): Longitude in WGS-84 degrees of the eastern boundary of the box Example: 30.0.
            west (float): Longitude in WGS-84 degrees of the western boundary of the box. Example: 30.0.
            exceptions (list[BoundingBoxArea | CorridorArea | EncodedCorridorArea | EncodedPolygonArea | PolygonArea] |
                Unset): Optional list of areas to exclude from avoidance.
    """

    type_: str
    north: float
    south: float
    east: float
    west: float
    exceptions: (
        list[
            BoundingBoxArea
            | CorridorArea
            | EncodedCorridorArea
            | EncodedPolygonArea
            | PolygonArea
        ]
        | Unset
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.bounding_box_area import BoundingBoxArea
        from ..models.corridor_area import CorridorArea
        from ..models.encoded_polygon_area import EncodedPolygonArea
        from ..models.polygon_area import PolygonArea

        type_ = self.type_

        north = self.north

        south = self.south

        east = self.east

        west = self.west

        exceptions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.exceptions, Unset):
            exceptions = []
            for exceptions_item_data in self.exceptions:
                exceptions_item: dict[str, Any]
                if isinstance(exceptions_item_data, BoundingBoxArea):
                    exceptions_item = exceptions_item_data.to_dict()
                elif isinstance(exceptions_item_data, PolygonArea):
                    exceptions_item = exceptions_item_data.to_dict()
                elif isinstance(exceptions_item_data, EncodedPolygonArea):
                    exceptions_item = exceptions_item_data.to_dict()
                elif isinstance(exceptions_item_data, CorridorArea):
                    exceptions_item = exceptions_item_data.to_dict()
                else:
                    exceptions_item = exceptions_item_data.to_dict()

                exceptions.append(exceptions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "north": north,
            "south": south,
            "east": east,
            "west": west,
        })
        if exceptions is not UNSET:
            field_dict["exceptions"] = exceptions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.bounding_box_area import BoundingBoxArea
        from ..models.corridor_area import CorridorArea
        from ..models.encoded_corridor_area import EncodedCorridorArea
        from ..models.encoded_polygon_area import EncodedPolygonArea
        from ..models.polygon_area import PolygonArea

        d = dict(src_dict)
        type_ = d.pop("type")

        north = d.pop("north")

        south = d.pop("south")

        east = d.pop("east")

        west = d.pop("west")

        _exceptions = d.pop("exceptions", UNSET)
        exceptions: (
            list[
                BoundingBoxArea
                | CorridorArea
                | EncodedCorridorArea
                | EncodedPolygonArea
                | PolygonArea
            ]
            | Unset
        ) = UNSET
        if _exceptions is not UNSET:
            exceptions = []
            for exceptions_item_data in _exceptions:

                def _parse_exceptions_item(
                    data: object,
                ) -> (
                    BoundingBoxArea
                    | CorridorArea
                    | EncodedCorridorArea
                    | EncodedPolygonArea
                    | PolygonArea
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_type_0 = BoundingBoxArea.from_dict(data)

                        return componentsschemas_area_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_type_1 = PolygonArea.from_dict(data)

                        return componentsschemas_area_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_type_2 = EncodedPolygonArea.from_dict(
                            data
                        )

                        return componentsschemas_area_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_type_3 = CorridorArea.from_dict(data)

                        return componentsschemas_area_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_area_type_4 = EncodedCorridorArea.from_dict(data)

                    return componentsschemas_area_type_4

                exceptions_item = _parse_exceptions_item(exceptions_item_data)

                exceptions.append(exceptions_item)

        bounding_box_area_with_exceptions = cls(
            type_=type_,
            north=north,
            south=south,
            east=east,
            west=west,
            exceptions=exceptions,
        )

        bounding_box_area_with_exceptions.additional_properties = d
        return bounding_box_area_with_exceptions

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.postal_code_details_usps_zip_plus_4_postal_code_type import (
    PostalCodeDetailsUspsZipPlus4PostalCodeType,
)
from ..models.postal_code_details_usps_zip_plus_4_postal_entity import (
    PostalCodeDetailsUspsZipPlus4PostalEntity,
)
from ..models.postal_code_details_usps_zip_plus_4_record_type_code import (
    PostalCodeDetailsUspsZipPlus4RecordTypeCode,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PostalCodeDetailsUspsZipPlus4")


@_attrs_define
class PostalCodeDetailsUspsZipPlus4:
    """
    Attributes:
        postal_code (str): The postal code.
        postal_entity (PostalCodeDetailsUspsZipPlus4PostalEntity | Unset): The postal entity. This could be a
            governmental authority, a regulatory authority, or a designated postal operator.

            Description of supported values:

            - `USPS`: The USPS postal code system.
        postal_code_type (PostalCodeDetailsUspsZipPlus4PostalCodeType | Unset): The postal code type. Currently
            supported values: `ZIP` and `ZIP+4`

            Description of supported values:

            - `ZIP+4`: Nine-digit code that identifies a small geographic delivery area that is serviceable by a single
            carrier; appears in the last line of the address on a mail piece.
        record_type_code (PostalCodeDetailsUspsZipPlus4RecordTypeCode | Unset): The USPS ZIP+4 Record Type Code.

            Description of supported values:

            - `F`: Firm
            - `G`: General delivery
            - `H`: High-rise
            - `P`: PO Box
            - `R`: Rural route/contract
            - `S`: Street
    """

    postal_code: str
    postal_entity: PostalCodeDetailsUspsZipPlus4PostalEntity | Unset = UNSET
    postal_code_type: PostalCodeDetailsUspsZipPlus4PostalCodeType | Unset = UNSET
    record_type_code: PostalCodeDetailsUspsZipPlus4RecordTypeCode | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        postal_code = self.postal_code

        postal_entity: str | Unset = UNSET
        if not isinstance(self.postal_entity, Unset):
            postal_entity = self.postal_entity.value

        postal_code_type: str | Unset = UNSET
        if not isinstance(self.postal_code_type, Unset):
            postal_code_type = self.postal_code_type.value

        record_type_code: str | Unset = UNSET
        if not isinstance(self.record_type_code, Unset):
            record_type_code = self.record_type_code.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "postalCode": postal_code,
        })
        if postal_entity is not UNSET:
            field_dict["postalEntity"] = postal_entity
        if postal_code_type is not UNSET:
            field_dict["postalCodeType"] = postal_code_type
        if record_type_code is not UNSET:
            field_dict["recordTypeCode"] = record_type_code

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        postal_code = d.pop("postalCode")

        _postal_entity = d.pop("postalEntity", UNSET)
        postal_entity: PostalCodeDetailsUspsZipPlus4PostalEntity | Unset
        if isinstance(_postal_entity, Unset):
            postal_entity = UNSET
        else:
            postal_entity = PostalCodeDetailsUspsZipPlus4PostalEntity(_postal_entity)

        _postal_code_type = d.pop("postalCodeType", UNSET)
        postal_code_type: PostalCodeDetailsUspsZipPlus4PostalCodeType | Unset
        if isinstance(_postal_code_type, Unset):
            postal_code_type = UNSET
        else:
            postal_code_type = PostalCodeDetailsUspsZipPlus4PostalCodeType(
                _postal_code_type
            )

        _record_type_code = d.pop("recordTypeCode", UNSET)
        record_type_code: PostalCodeDetailsUspsZipPlus4RecordTypeCode | Unset
        if isinstance(_record_type_code, Unset):
            record_type_code = UNSET
        else:
            record_type_code = PostalCodeDetailsUspsZipPlus4RecordTypeCode(
                _record_type_code
            )

        postal_code_details_usps_zip_plus_4 = cls(
            postal_code=postal_code,
            postal_entity=postal_entity,
            postal_code_type=postal_code_type,
            record_type_code=record_type_code,
        )

        postal_code_details_usps_zip_plus_4.additional_properties = d
        return postal_code_details_usps_zip_plus_4

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Driver")


@_attrs_define
class Driver:
    """Driver parameters to be used for calculating routes with automatically added
    rest stops.

        Attributes:
            schedule (str | Unset): A comma separated list of duration pairs in seconds. Each pair is two positive non-zero
                integers.
                The first specifies the maximum allowed drive duration before stopping to rest, and the second the resting
                duration before continuing driving.

                Format `driveDuration,restDuration[,driveDuration,restDuration]...`

                * driveDuration - duration in seconds, describes maximum driving duration allowed in a route's section
                  till the next stop or route end. Minimum value is 240.
                * restDuration - duration in seconds, describes minimum resting duration required before continuing driving

                The routing engine adds necessary stops for rest to fulfill driver work hours requirements.
                If the route duration exceeds the defined drive-rest sequence the remaining route is added as the last drive
                section
                (or several sections if there are `via` waypoints on the remaining route) with critical notice
                `travelTimeExceedsDriverWorkHours`.

                **Limitations**:
                * `driver[schedule]` is only supported for `truck`, `bus` and `privateBus` transport modes.
                * `driver[schedule]` is not supported in combination with any of these parameters: `arrivalTime`, `ev[*]`.
                * `driver[schedule]` doesn't support route alternatives. So `alternatives` parameter value is ignored in
                combination with `driver[schedule]`.
                * Ferry travel, boarding, and deboarding durations are neither considered part of drive duration nor rest
                duration unless ferry
                  travel duration is below 15 minutes. In this case, ferry travel duration is considered as drive duration.
                * Via waypoint stop-durations are neither considered part of drive duration nor rest duration.
                * Routing requires a minimum driving duration (`driveDuration`) of 240 seconds before a rest stop.
    """

    schedule: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        schedule = self.schedule

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if schedule is not UNSET:
            field_dict["schedule"] = schedule

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        schedule = d.pop("schedule", UNSET)

        driver = cls(
            schedule=schedule,
        )

        driver.additional_properties = d
        return driver

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.fuel_station_fuel_types_item import FuelStationFuelTypesItem
from ..models.fuel_station_minimum_truck_class import FuelStationMinimumTruckClass
from ..types import UNSET, Unset

T = TypeVar("T", bound="FuelStation")


@_attrs_define
class FuelStation:
    """
    Attributes:
        minimum_truck_class (FuelStationMinimumTruckClass | Unset): **BETA, RESTRICTED**

            All place results returned with a category `700-7600-0116`, `700-7600-0000`, `700-7600-0444`, or `700-7900-0132`
            that serves truck fuel types with minimum supported truck class specified in this parameter.
            See the related [Appendix](https://www.here.com/docs/bundle/geocoding-and-search-api-developer-
            guide/page/topics-places/places-category-system-full.html#700-business-and-services)
            for details about those place category IDs.

            Caution: Note that this filter will be applied to place results regardless of the query intent.
            For example Places with both a gas station and a grocery category will be returned to a "grocery" query
            only if the place truck classes complies with the values of the query `fuelStation[minimumTruckClass]` values.

            Description of supported values:

            - **BETA, RESTRICTED** `heavy`: Medium and Heavy trucks are allowed to fuel in the gas station
            - **BETA, RESTRICTED** `medium`: Only medium size truck is allowed to fuel in the gas station Example: medium.
        fuel_types (list[FuelStationFuelTypesItem] | Unset): **BETA, RESTRICTED**

            All place results returned with a category `700-7600-0116`, `700-7600-0000`, `700-7600-0444`, or `700-7900-0132`
            serve at least one fuel type specified in the comma-separated values of this parameter.
            See the related [Appendix](https://www.here.com/docs/bundle/geocoding-and-search-api-developer-
            guide/page/topics-places/places-category-system-full.html#700-business-and-services)
            for details about those place category IDs.

            Caution: Note that this filter will be applied to place results regardless of the query intent.
            For example Places with both a gas station and a grocery category will be returned to a "grocery" query
            only if the place fuel types complies with the values of the query `fuelStation[fuelTypes]` values.

            Description of supported values:

            - **BETA, RESTRICTED** `biodiesel`: bio-diesel
            - **BETA, RESTRICTED** `cng`: compressed natural gas (CNG)
            - **BETA, RESTRICTED** `diesel`: diesel
            - **BETA, RESTRICTED** `diesel_with_additives`: diesel with additives
            - **BETA, RESTRICTED** `e10`: E10 (10% ethanol)
            - **BETA, RESTRICTED** `e20`: E20 (20% ethanol)
            - **BETA, RESTRICTED** `e85`: E85 (minimum 70% ethanol blended gasoline)
            - **BETA, RESTRICTED** `ethanol`: ethanol fuel (when specific type is not known, such as E10, E85)
            - **BETA, RESTRICTED** `ethanol_with_additives`: ethanol with additives
            - **BETA, RESTRICTED** `gasoline`: gasoline
            - **BETA, RESTRICTED** `hvo`: hydrotreated vegetable oil fuel
            - **BETA, RESTRICTED** `hydrogen`: hydrogen
            - **BETA, RESTRICTED** `lng`: liquefied natural gas (LNG)
            - **BETA, RESTRICTED** `lpg`: liquefied petroleum gas (LPG)
            - **BETA, RESTRICTED** `midgrade`: midgrade octane rating
            - **BETA, RESTRICTED** `octane_100`: fuel that consists of 100% octane
            - **BETA, RESTRICTED** `octane_87`: fuel that consists of an octane / gasoline blend with 87% octane / 13%
            gasoline
            - **BETA, RESTRICTED** `octane_89`: fuel that consists of an octane / gasoline blend with 89% octane / 11%
            gasoline
            - **BETA, RESTRICTED** `octane_90`: fuel that consists of an octane / gasoline blend with 90% octane / 10%
            gasoline
            - **BETA, RESTRICTED** `octane_91`: fuel that consists of an octane / gasoline blend with 91% octane / 9%
            gasoline
            - **BETA, RESTRICTED** `octane_92`: fuel that consists of an octane / gasoline blend with 92% octane / 8%
            gasoline
            - **BETA, RESTRICTED** `octane_93`: fuel that consists of octane / gasoline blend with 93% octane / 7% gasoline
            - **BETA, RESTRICTED** `octane_95`: fuel that consists of an octane / gasoline blend with 95% octane / 5%
            gasoline
            - **BETA, RESTRICTED** `octane_98`:  fuel that consists of an octane / gasoline blend with 98% octane / 2%
            gasoline
            - **BETA, RESTRICTED** `premium`: premium octane rating
            - **BETA, RESTRICTED** `regular`: regular octance rating
            - **BETA, RESTRICTED** `truck_cng`: compressed natural gas (CNG) fuel for truck
            - **BETA, RESTRICTED** `truck_diesel`: diesel fuel for truck
            - **BETA, RESTRICTED** `truck_hydrogen`: hydrogen fuel for truck
            - **BETA, RESTRICTED** `truck_lng`: liquefied natural gas (LNG) fuel for truck Example: ['biodiesel', 'diesel',
            'truck_diesel'].
    """

    minimum_truck_class: FuelStationMinimumTruckClass | Unset = UNSET
    fuel_types: list[FuelStationFuelTypesItem] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        minimum_truck_class: str | Unset = UNSET
        if not isinstance(self.minimum_truck_class, Unset):
            minimum_truck_class = self.minimum_truck_class.value

        fuel_types: list[str] | Unset = UNSET
        if not isinstance(self.fuel_types, Unset):
            fuel_types = []
            for fuel_types_item_data in self.fuel_types:
                fuel_types_item = fuel_types_item_data.value
                fuel_types.append(fuel_types_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if minimum_truck_class is not UNSET:
            field_dict["minimumTruckClass"] = minimum_truck_class
        if fuel_types is not UNSET:
            field_dict["fuelTypes"] = fuel_types

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _minimum_truck_class = d.pop("minimumTruckClass", UNSET)
        minimum_truck_class: FuelStationMinimumTruckClass | Unset
        if isinstance(_minimum_truck_class, Unset):
            minimum_truck_class = UNSET
        else:
            minimum_truck_class = FuelStationMinimumTruckClass(_minimum_truck_class)

        _fuel_types = d.pop("fuelTypes", UNSET)
        fuel_types: list[FuelStationFuelTypesItem] | Unset = UNSET
        if _fuel_types is not UNSET:
            fuel_types = []
            for fuel_types_item_data in _fuel_types:
                fuel_types_item = FuelStationFuelTypesItem(fuel_types_item_data)

                fuel_types.append(fuel_types_item)

        fuel_station = cls(
            minimum_truck_class=minimum_truck_class,
            fuel_types=fuel_types,
        )

        fuel_station.additional_properties = d
        return fuel_station

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="ExitInfo")


@_attrs_define
class ExitInfo:
    """Exit information attached to an offset action

    Example:
        {'exit': {'number': [{'value': '15', 'language': 'de'}]}}

    Attributes:
        number (list[LocalizedString] | Unset): Number of the exit (e.g. '18')
    """

    number: list[LocalizedString] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        number: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.number, Unset):
            number = []
            for number_item_data in self.number:
                number_item = number_item_data.to_dict()
                number.append(number_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if number is not UNSET:
            field_dict["number"] = number

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        _number = d.pop("number", UNSET)
        number: list[LocalizedString] | Unset = UNSET
        if _number is not UNSET:
            number = []
            for number_item_data in _number:
                number_item = LocalizedString.from_dict(number_item_data)

                number.append(number_item)

        exit_info = cls(
            number=number,
        )

        exit_info.additional_properties = d
        return exit_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.max_speed_type_1 import MaxSpeedType1
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dynamic_speed_info import DynamicSpeedInfo
    from ..models.localized_route_number import LocalizedRouteNumber
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="PedestrianSpan")


@_attrs_define
class PedestrianSpan:
    r"""Span attached to a `Section` describing pedestrian content.

    Attributes:
        offset (int | Unset): Offset of a coordinate in the section's polyline.
        length (int | Unset): Distance in meters. Example: 189.
        duration (int | Unset): Duration in seconds. Example: 198.
        street_attributes (list[str] | Unset): `StreetAttributes` is applied to a span of a route section and describes
            attribute flags of a street.
            * `rightDrivingSide`: Do vehicles have to drive on the right-hand side of the road or the left-hand side.
            * `dirtRoad`: This part of the route has an un-paved surface.
            * `tunnel`: This part of the route is a tunnel.
            * `bridge`: This part of the route is a bridge.
            * `ramp`: This part of the route is a ramp (usually connecting to/from/between highways).
            * `motorway`: This part of the route is a controlled access road (usually highways with motorway sign).
            * `roundabout`: This part of the route is a roundabout.
            * `underConstruction`: This part of the route is under construction.
            * `dividedRoad`: This part of the route uses a road with a physical or legal divider in the middle.
            * `privateRoad`: This part of the route uses a privately owned road.
            * `controlledAccessHighway`: This part of the route is a either controlled access or limited access road or
            both.
            * `builtUpArea`: This part of the route most likely passes through a built-up area.

            As it is possible that new street attributes are supported in the future, unknown street attributes should be
            ignored.
        walk_attributes (list[str] | Unset): Accessibility and walk-related attribute flags.

            * `stairs`: This part of the route is a staircase.
            * `park`: This part of the route is in a park.
            * `indoor`: This part of the route is inside a venue.
            * `open`: This part of the route is open to walking.
            * `noThrough`: This part of the route can only be traversed if origin, destination or any via waypoint is
            located there.
            * `tollRoad`: Access to this part of the route is restricted with a fee (or toll).

            As it is possible that new attributes are supported in the future, unknown attributes should be ignored.
        car_attributes (list[str] | Unset): Car specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describes access flags of a street.
            * `open`:  A part of the route that is open to cars.
            * `noThrough`:  A part of the route that can only be traversed if origin, destination or any via waypoint is
            located there.
            * `tollRoad`: Access to this part of the route is restricted with a fee (or toll).

            As it is possible that new access attributes are supported in the future, unknown access attributes should be
            ignored.
        truck_attributes (list[str] | Unset): Truck specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describes access flags of a street.
            * `open`: This part of the route is open to trucks.
            * `noThrough`: This part of the route can only be traversed if origin, destination or any via waypoint is
            located there.
            * `tollRoad`: This part of the route is restricted with a fee (or toll).

            As it is possible that new access attributes are supported in the future, unknown access attributes should be
            ignored.
        scooter_attributes (list[str] | Unset): Scooter specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describes access flags of a street.
            * `open`: This part of the route is open to scooters.

            As it is possible that new access attributes are supported in the future, unknown access attributes should be
            ignored.
        names (list[LocalizedString] | Unset): Designated name for the span (e.g. a street name or a transport name)
        route_numbers (list[LocalizedRouteNumber] | Unset): Designated route name or number of the span (e.g. 'M25')
        country_code (str | Unset): ISO-3166-1 alpha-3 code Example: FRA.
        state_code (str | Unset): The second part of an ISO-3166-2 code (e.g., `TX` from `USA-TX`) consists of up to
            three alphanumeric characters.
            It is used to identify the principal subdivisions (e.g., provinces or states) of a country in conjunction with a
            CountryCode

            Note: State codes may not be available in some countries.
        functional_class (int | Unset): Functional class defines a hierarchical network used to determine a logical and
            efficient route. The following classifications are used:

            * `1`: Roads that allow for high volume, maximum speed traffic movement between and through major metropolitan
            areas.
            * `2`: Roads that are used to channel traffic to functional class 1 roads for travel between and through cities
            in the shortest amount of time.
            * `3`: Roads that intersect functional class 2 roads and provide a high volume of traffic movement at a lower
            level of mobility than functional class 2 roads.
            * `4`: Roads that provide for a high volume of traffic movement at moderate speeds between neighborhoods.
            * `5`: Roads with volume and traffic movement below the level of any other functional class.
        speed_limit (float | Unset): Speed in meters per second
        max_speed (float | MaxSpeedType1 | Unset): Speed in meters per second, or "unlimited" indicating that the speed
            is unlimited, e.g., on a German autobahn
        dynamic_speed_info (DynamicSpeedInfo | Unset): Describes dynamic speed information, such as traffic speed,
            estimated speed without traffic, and turn time.
        segment_id (str | Unset): **NOTE:** Attribute segmentId is deprecated. Use segmentRef instead.

            The directed topology segment id including prefix (e.g '+here:cm:segment:').

            The id consists of two parts.
            * The direction ('+' or '-')
            * followed by the topology segment id (a unique identifier within the HERE platform catalogs).

            The direction specifies whether the route is using the segment in its canonical direction ('+' aka traveling
            along the geometry's direction), or against it ('-' aka traveling against the geometry's direction).
        segment_ref (str | Unset): A reference to the HMC topology segment used in this span.

            The standard representation of a segment reference has the following structure:
            {catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{segmentId}(#{direction}({startOffset}..{endOffset})?)?

            The individual parts are:
            * catalogHrn: The HERE Resource Name that identifies the source catalog of the segment, example:
            hrn:here:data::olp-here:rib-2
            * catalogVersion: The catalog version
            * layerId (optional): The layer inside the catalog where the segment can be found, example: topology-geometry
            * tileId: The HERE tile key of the partition/tile where the segment is located in the given version of the
            catalog. This can be on a lower level than the actual segment is stored at (for example, the provided tile ID
            can be on level 14, despite topology-geometry partitions being tiled at level 12). The level of a HERE tile key
            is indicated by the position of the highest set bit in binary representation. Since the HERE tile key represents
            a morton code of the x and y portion of the Tile ID, the level 12 tile ID can be retrieved from the level 14
            tile ID by removing the 4 least significant bits (or 2 bits per level) or 1 hexadecimal digit. For example, the
            level 14 tile 377894441 is included in the level 12 tile 23618402 (377894441<sub>10</sub> =
            16863629<sub>16</sub> &rightarrow; 1686362<sub>16</sub> = 23618402<sub>10</sub>)
            * segmentId: The identifier of the referenced topology segment inside the catalog, example:
            here:cm:segment:84905195
            * direction (optional): Either '*' for undirected or bidirectional, '+' for positive direction, '-' for negative
            direction, or '?' for unknown direction (not used by the routing service)
            * startOffset/endOffset (optional): The start- and end offset are non-negative numbers between 0 and 1,
            representing the start and end of the referenced range using a proportion of the length of the segment. 0
            represents the start and 1 the end of the segment, relative to the indicated direction (or positive direction in
            case of undirected segments). Example: 0.7..1

            Example of a segment reference in standard representation:
            hrn:here:data::olp-here:rib-2:1363::377894441:here:cm:segment:84905195#+0.7..1

            The segment references can also be provided in a compact representation, to reduce the response size. In the
            compact representation, some parts are replaced by placeholders, which can be resolved using the refReplacements
            dictionary in the parent section.
            The placeholder format is ```\$\d+``` and needs to be surrounded by colons or string start/end. It can be
            captured with the following regular expression: ```(^|:)\$\d+(:|$)/``` .

            Example of the segment reference previously mentioned in compact representation:
            $0:377894441:$1:84905195#+0.7..1
            With the corresponding refReplacements:
            "refReplacements": {
              "0": "hrn:here:data::olp-here:rib-2:1363:",
              "1": "here:cm:segment"
            }
        base_duration (int | Unset): Duration in seconds. Example: 198.
        typical_duration (int | Unset): Duration in seconds. Example: 198.
        incidents (list[int] | Unset): A list of indices into the incident array of the parent section.
            References all incidents that apply to the span.
    """

    offset: int | Unset = UNSET
    length: int | Unset = UNSET
    duration: int | Unset = UNSET
    street_attributes: list[str] | Unset = UNSET
    walk_attributes: list[str] | Unset = UNSET
    car_attributes: list[str] | Unset = UNSET
    truck_attributes: list[str] | Unset = UNSET
    scooter_attributes: list[str] | Unset = UNSET
    names: list[LocalizedString] | Unset = UNSET
    route_numbers: list[LocalizedRouteNumber] | Unset = UNSET
    country_code: str | Unset = UNSET
    state_code: str | Unset = UNSET
    functional_class: int | Unset = UNSET
    speed_limit: float | Unset = UNSET
    max_speed: float | MaxSpeedType1 | Unset = UNSET
    dynamic_speed_info: DynamicSpeedInfo | Unset = UNSET
    segment_id: str | Unset = UNSET
    segment_ref: str | Unset = UNSET
    base_duration: int | Unset = UNSET
    typical_duration: int | Unset = UNSET
    incidents: list[int] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        offset = self.offset

        length = self.length

        duration = self.duration

        street_attributes: list[str] | Unset = UNSET
        if not isinstance(self.street_attributes, Unset):
            street_attributes = self.street_attributes

        walk_attributes: list[str] | Unset = UNSET
        if not isinstance(self.walk_attributes, Unset):
            walk_attributes = self.walk_attributes

        car_attributes: list[str] | Unset = UNSET
        if not isinstance(self.car_attributes, Unset):
            car_attributes = self.car_attributes

        truck_attributes: list[str] | Unset = UNSET
        if not isinstance(self.truck_attributes, Unset):
            truck_attributes = self.truck_attributes

        scooter_attributes: list[str] | Unset = UNSET
        if not isinstance(self.scooter_attributes, Unset):
            scooter_attributes = self.scooter_attributes

        names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.names, Unset):
            names = []
            for names_item_data in self.names:
                names_item = names_item_data.to_dict()
                names.append(names_item)

        route_numbers: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.route_numbers, Unset):
            route_numbers = []
            for route_numbers_item_data in self.route_numbers:
                route_numbers_item = route_numbers_item_data.to_dict()
                route_numbers.append(route_numbers_item)

        country_code = self.country_code

        state_code = self.state_code

        functional_class = self.functional_class

        speed_limit = self.speed_limit

        max_speed: float | str | Unset
        if isinstance(self.max_speed, Unset):
            max_speed = UNSET
        elif isinstance(self.max_speed, MaxSpeedType1):
            max_speed = self.max_speed.value
        else:
            max_speed = self.max_speed

        dynamic_speed_info: dict[str, Any] | Unset = UNSET
        if not isinstance(self.dynamic_speed_info, Unset):
            dynamic_speed_info = self.dynamic_speed_info.to_dict()

        segment_id = self.segment_id

        segment_ref = self.segment_ref

        base_duration = self.base_duration

        typical_duration = self.typical_duration

        incidents: list[int] | Unset = UNSET
        if not isinstance(self.incidents, Unset):
            incidents = self.incidents

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if offset is not UNSET:
            field_dict["offset"] = offset
        if length is not UNSET:
            field_dict["length"] = length
        if duration is not UNSET:
            field_dict["duration"] = duration
        if street_attributes is not UNSET:
            field_dict["streetAttributes"] = street_attributes
        if walk_attributes is not UNSET:
            field_dict["walkAttributes"] = walk_attributes
        if car_attributes is not UNSET:
            field_dict["carAttributes"] = car_attributes
        if truck_attributes is not UNSET:
            field_dict["truckAttributes"] = truck_attributes
        if scooter_attributes is not UNSET:
            field_dict["scooterAttributes"] = scooter_attributes
        if names is not UNSET:
            field_dict["names"] = names
        if route_numbers is not UNSET:
            field_dict["routeNumbers"] = route_numbers
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if state_code is not UNSET:
            field_dict["stateCode"] = state_code
        if functional_class is not UNSET:
            field_dict["functionalClass"] = functional_class
        if speed_limit is not UNSET:
            field_dict["speedLimit"] = speed_limit
        if max_speed is not UNSET:
            field_dict["maxSpeed"] = max_speed
        if dynamic_speed_info is not UNSET:
            field_dict["dynamicSpeedInfo"] = dynamic_speed_info
        if segment_id is not UNSET:
            field_dict["segmentId"] = segment_id
        if segment_ref is not UNSET:
            field_dict["segmentRef"] = segment_ref
        if base_duration is not UNSET:
            field_dict["baseDuration"] = base_duration
        if typical_duration is not UNSET:
            field_dict["typicalDuration"] = typical_duration
        if incidents is not UNSET:
            field_dict["incidents"] = incidents

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.dynamic_speed_info import DynamicSpeedInfo
        from ..models.localized_route_number import LocalizedRouteNumber
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        offset = d.pop("offset", UNSET)

        length = d.pop("length", UNSET)

        duration = d.pop("duration", UNSET)

        street_attributes = cast(list[str], d.pop("streetAttributes", UNSET))

        walk_attributes = cast(list[str], d.pop("walkAttributes", UNSET))

        car_attributes = cast(list[str], d.pop("carAttributes", UNSET))

        truck_attributes = cast(list[str], d.pop("truckAttributes", UNSET))

        scooter_attributes = cast(list[str], d.pop("scooterAttributes", UNSET))

        _names = d.pop("names", UNSET)
        names: list[LocalizedString] | Unset = UNSET
        if _names is not UNSET:
            names = []
            for names_item_data in _names:
                names_item = LocalizedString.from_dict(names_item_data)

                names.append(names_item)

        _route_numbers = d.pop("routeNumbers", UNSET)
        route_numbers: list[LocalizedRouteNumber] | Unset = UNSET
        if _route_numbers is not UNSET:
            route_numbers = []
            for route_numbers_item_data in _route_numbers:
                route_numbers_item = LocalizedRouteNumber.from_dict(
                    route_numbers_item_data
                )

                route_numbers.append(route_numbers_item)

        country_code = d.pop("countryCode", UNSET)

        state_code = d.pop("stateCode", UNSET)

        functional_class = d.pop("functionalClass", UNSET)

        speed_limit = d.pop("speedLimit", UNSET)

        def _parse_max_speed(data: object) -> float | MaxSpeedType1 | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                componentsschemas_max_speed_type_1 = MaxSpeedType1(data)

                return componentsschemas_max_speed_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(float | MaxSpeedType1 | Unset, data)

        max_speed = _parse_max_speed(d.pop("maxSpeed", UNSET))

        _dynamic_speed_info = d.pop("dynamicSpeedInfo", UNSET)
        dynamic_speed_info: DynamicSpeedInfo | Unset
        if isinstance(_dynamic_speed_info, Unset):
            dynamic_speed_info = UNSET
        else:
            dynamic_speed_info = DynamicSpeedInfo.from_dict(_dynamic_speed_info)

        segment_id = d.pop("segmentId", UNSET)

        segment_ref = d.pop("segmentRef", UNSET)

        base_duration = d.pop("baseDuration", UNSET)

        typical_duration = d.pop("typicalDuration", UNSET)

        incidents = cast(list[int], d.pop("incidents", UNSET))

        pedestrian_span = cls(
            offset=offset,
            length=length,
            duration=duration,
            street_attributes=street_attributes,
            walk_attributes=walk_attributes,
            car_attributes=car_attributes,
            truck_attributes=truck_attributes,
            scooter_attributes=scooter_attributes,
            names=names,
            route_numbers=route_numbers,
            country_code=country_code,
            state_code=state_code,
            functional_class=functional_class,
            speed_limit=speed_limit,
            max_speed=max_speed,
            dynamic_speed_info=dynamic_speed_info,
            segment_id=segment_id,
            segment_ref=segment_ref,
            base_duration=base_duration,
            typical_duration=typical_duration,
            incidents=incidents,
        )

        pedestrian_span.additional_properties = d
        return pedestrian_span

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TruckAmenityGenericType(str, Enum):
    CARWASH = "carWash"
    CHEMICALTOILETDISPOSAL = "chemicalToiletDisposal"
    CONVENIENCESTORE = "convenienceStore"
    HIGHCANOPY = "highCanopy"
    IDLEREDUCTION = "idleReduction"
    NIGHTPARKINGONLY = "nightParkingOnly"
    PAIDPARKING = "paidParking"
    PARKING = "parking"
    POWERSUPPLY = "powerSupply"
    RESERVABLEPARKING = "reservableParking"
    SECUREPARKING = "secureParking"
    TRUCKSCALES = "truckScales"
    TRUCKSERVICE = "truckService"
    TRUCKSTOP = "truckStop"
    TRUCKWASH = "truckWash"
    WIFI = "wifi"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetGeocodeShowTranslationsItem(str, Enum):
    CITY = "city"
    COUNTY = "county"
    DISTRICT = "district"
    STATE = "state"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.browse_result_item_address_block_type import (
    BrowseResultItemAddressBlockType,
)
from ..models.browse_result_item_administrative_area_type import (
    BrowseResultItemAdministrativeAreaType,
)
from ..models.browse_result_item_locality_type import BrowseResultItemLocalityType
from ..models.browse_result_item_result_type import BrowseResultItemResultType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.access_restriction_attributes import AccessRestrictionAttributes
    from ..models.address import Address
    from ..models.category import Category
    from ..models.chain import Chain
    from ..models.contact_information import ContactInformation
    from ..models.display_response_coordinate import DisplayResponseCoordinate
    from ..models.extended_access_point import ExtendedAccessPoint
    from ..models.extended_attribute import ExtendedAttribute
    from ..models.map_view import MapView
    from ..models.media import Media
    from ..models.opening_hours import OpeningHours
    from ..models.phonemes_section import PhonemesSection
    from ..models.street_info import StreetInfo
    from ..models.supplier_reference import SupplierReference
    from ..models.time_zone_info import TimeZoneInfo


T = TypeVar("T", bound="BrowseResultItem")


@_attrs_define
class BrowseResultItem:
    """
    Attributes:
        title (str): The localized display name of this result item.
        id (str): The unique identifier for the result item. This ID can be used for a Look Up by ID search as well.
        address (Address):
        political_view (str | Unset): ISO3 country code of the item political view (default for international). This
            response element is populated when the politicalView parameter is set in the query
        result_type (BrowseResultItemResultType | Unset):
            Type of the result item.

            Note: `addressBlock` result item is either a block or subblock.

            `resultType` values can get added to the list without further notice.
        address_block_type (BrowseResultItemAddressBlockType | Unset):
            Type of the address block result if the `resultType` field is set to `addressBlock`.

            `addressBlockType` values can get added to the list without further notice.
        locality_type (BrowseResultItemLocalityType | Unset):
            Type of the locality result if the `resultType` field is set to `locality`.

            `localityType` values can get added to the list without further notice.
        administrative_area_type (BrowseResultItemAdministrativeAreaType | Unset):
            Type of the administrative area result if the `resultType` field is set to `administrativeArea`.

            `administrativeAreaType` values can get added to the list without further notice.
        position (DisplayResponseCoordinate | Unset):
        access (list[ExtendedAccessPoint] | Unset): Coordinates of the response item on a HERE map navigable link (for
            example, for driving or walking).

            When a response item is of type `"place"`, it can contain more than one access object.
            Each access object contains a position (`lat`, `lng`), with potentially a type (field `type`),
            a textual label (field `label`) and a `primary` boolean field set to true for the main access to the place.

            The first object in the `access` array is the primary access to the place.

            Response items of other types than `"place"` have a maximum of one access position.
        drive_through (bool | Unset): Set to `true` if the related business allows customers to purchase products
            (or use the service provided by the business) without leaving their cars.
        distance (int | Unset): The distance "as the crow flies" from the search center to this result item in meters.
            For example: "172039".

            When searching along a route this is the distance along the route plus the distance from the route polyline to
            this result item. Example: 172039.
        excursion_distance (int | Unset): Two times the distance from the polyline to this result item in meters while
            searching along the route
        map_view (MapView | Unset):
        categories (list[Category] | Unset): The list of categories assigned to this place.
        chains (list[Chain] | Unset): The list of chains assigned to this place.
        references (list[SupplierReference] | Unset): The list of supplier references available for this place.
        food_types (list[Category] | Unset): The list of food types assigned to this place.
        contacts (list[ContactInformation] | Unset): Contact information like phone, email, WWW.
        opening_hours (list[OpeningHours] | Unset): A list of hours during which the place is open for business.
            This field is optional: When it is not present, it means that we are lacking data about the place opening hours.
            Days without opening hours have to be considered as closed.
        time_zone (TimeZoneInfo | Unset):
        extended (ExtendedAttribute | Unset):
        phonemes (PhonemesSection | Unset):
        media (Media | Unset):
        street_info (list[StreetInfo] | Unset): Street Details (only rendered if `show=streetInfo` is provided.)
        access_restrictions (list[AccessRestrictionAttributes] | Unset): A list of access restrictions related to a
            `place` result item.

            A place response item has a restricted access when the place is not publicly accessible without a specific
            authorization, membership or customer relationship.
            Examples: the place is for customers or visitors only.

            This field is optional: When it is not present, it means that the place response item has no known access
            restriction. Example: [{'categories': [{'id': '800-8500-0177'}], 'restricted': True}].
    """

    title: str
    id: str
    address: Address
    political_view: str | Unset = UNSET
    result_type: BrowseResultItemResultType | Unset = UNSET
    address_block_type: BrowseResultItemAddressBlockType | Unset = UNSET
    locality_type: BrowseResultItemLocalityType | Unset = UNSET
    administrative_area_type: BrowseResultItemAdministrativeAreaType | Unset = UNSET
    position: DisplayResponseCoordinate | Unset = UNSET
    access: list[ExtendedAccessPoint] | Unset = UNSET
    drive_through: bool | Unset = UNSET
    distance: int | Unset = UNSET
    excursion_distance: int | Unset = UNSET
    map_view: MapView | Unset = UNSET
    categories: list[Category] | Unset = UNSET
    chains: list[Chain] | Unset = UNSET
    references: list[SupplierReference] | Unset = UNSET
    food_types: list[Category] | Unset = UNSET
    contacts: list[ContactInformation] | Unset = UNSET
    opening_hours: list[OpeningHours] | Unset = UNSET
    time_zone: TimeZoneInfo | Unset = UNSET
    extended: ExtendedAttribute | Unset = UNSET
    phonemes: PhonemesSection | Unset = UNSET
    media: Media | Unset = UNSET
    street_info: list[StreetInfo] | Unset = UNSET
    access_restrictions: list[AccessRestrictionAttributes] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        id = self.id

        address = self.address.to_dict()

        political_view = self.political_view

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        address_block_type: str | Unset = UNSET
        if not isinstance(self.address_block_type, Unset):
            address_block_type = self.address_block_type.value

        locality_type: str | Unset = UNSET
        if not isinstance(self.locality_type, Unset):
            locality_type = self.locality_type.value

        administrative_area_type: str | Unset = UNSET
        if not isinstance(self.administrative_area_type, Unset):
            administrative_area_type = self.administrative_area_type.value

        position: dict[str, Any] | Unset = UNSET
        if not isinstance(self.position, Unset):
            position = self.position.to_dict()

        access: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = []
            for access_item_data in self.access:
                access_item = access_item_data.to_dict()
                access.append(access_item)

        drive_through = self.drive_through

        distance = self.distance

        excursion_distance = self.excursion_distance

        map_view: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_view, Unset):
            map_view = self.map_view.to_dict()

        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        chains: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.chains, Unset):
            chains = []
            for chains_item_data in self.chains:
                chains_item = chains_item_data.to_dict()
                chains.append(chains_item)

        references: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.references, Unset):
            references = []
            for references_item_data in self.references:
                references_item = references_item_data.to_dict()
                references.append(references_item)

        food_types: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.food_types, Unset):
            food_types = []
            for food_types_item_data in self.food_types:
                food_types_item = food_types_item_data.to_dict()
                food_types.append(food_types_item)

        contacts: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.contacts, Unset):
            contacts = []
            for contacts_item_data in self.contacts:
                contacts_item = contacts_item_data.to_dict()
                contacts.append(contacts_item)

        opening_hours: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.opening_hours, Unset):
            opening_hours = []
            for opening_hours_item_data in self.opening_hours:
                opening_hours_item = opening_hours_item_data.to_dict()
                opening_hours.append(opening_hours_item)

        time_zone: dict[str, Any] | Unset = UNSET
        if not isinstance(self.time_zone, Unset):
            time_zone = self.time_zone.to_dict()

        extended: dict[str, Any] | Unset = UNSET
        if not isinstance(self.extended, Unset):
            extended = self.extended.to_dict()

        phonemes: dict[str, Any] | Unset = UNSET
        if not isinstance(self.phonemes, Unset):
            phonemes = self.phonemes.to_dict()

        media: dict[str, Any] | Unset = UNSET
        if not isinstance(self.media, Unset):
            media = self.media.to_dict()

        street_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.street_info, Unset):
            street_info = []
            for street_info_item_data in self.street_info:
                street_info_item = street_info_item_data.to_dict()
                street_info.append(street_info_item)

        access_restrictions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access_restrictions, Unset):
            access_restrictions = []
            for access_restrictions_item_data in self.access_restrictions:
                access_restrictions_item = access_restrictions_item_data.to_dict()
                access_restrictions.append(access_restrictions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "id": id,
            "address": address,
        })
        if political_view is not UNSET:
            field_dict["politicalView"] = political_view
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if address_block_type is not UNSET:
            field_dict["addressBlockType"] = address_block_type
        if locality_type is not UNSET:
            field_dict["localityType"] = locality_type
        if administrative_area_type is not UNSET:
            field_dict["administrativeAreaType"] = administrative_area_type
        if position is not UNSET:
            field_dict["position"] = position
        if access is not UNSET:
            field_dict["access"] = access
        if drive_through is not UNSET:
            field_dict["driveThrough"] = drive_through
        if distance is not UNSET:
            field_dict["distance"] = distance
        if excursion_distance is not UNSET:
            field_dict["excursionDistance"] = excursion_distance
        if map_view is not UNSET:
            field_dict["mapView"] = map_view
        if categories is not UNSET:
            field_dict["categories"] = categories
        if chains is not UNSET:
            field_dict["chains"] = chains
        if references is not UNSET:
            field_dict["references"] = references
        if food_types is not UNSET:
            field_dict["foodTypes"] = food_types
        if contacts is not UNSET:
            field_dict["contacts"] = contacts
        if opening_hours is not UNSET:
            field_dict["openingHours"] = opening_hours
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if extended is not UNSET:
            field_dict["extended"] = extended
        if phonemes is not UNSET:
            field_dict["phonemes"] = phonemes
        if media is not UNSET:
            field_dict["media"] = media
        if street_info is not UNSET:
            field_dict["streetInfo"] = street_info
        if access_restrictions is not UNSET:
            field_dict["accessRestrictions"] = access_restrictions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.access_restriction_attributes import AccessRestrictionAttributes
        from ..models.address import Address
        from ..models.category import Category
        from ..models.chain import Chain
        from ..models.contact_information import ContactInformation
        from ..models.display_response_coordinate import DisplayResponseCoordinate
        from ..models.extended_access_point import ExtendedAccessPoint
        from ..models.extended_attribute import ExtendedAttribute
        from ..models.map_view import MapView
        from ..models.media import Media
        from ..models.opening_hours import OpeningHours
        from ..models.phonemes_section import PhonemesSection
        from ..models.street_info import StreetInfo
        from ..models.supplier_reference import SupplierReference
        from ..models.time_zone_info import TimeZoneInfo

        d = dict(src_dict)
        title = d.pop("title")

        id = d.pop("id")

        address = Address.from_dict(d.pop("address"))

        political_view = d.pop("politicalView", UNSET)

        _result_type = d.pop("resultType", UNSET)
        result_type: BrowseResultItemResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = BrowseResultItemResultType(_result_type)

        _address_block_type = d.pop("addressBlockType", UNSET)
        address_block_type: BrowseResultItemAddressBlockType | Unset
        if isinstance(_address_block_type, Unset):
            address_block_type = UNSET
        else:
            address_block_type = BrowseResultItemAddressBlockType(_address_block_type)

        _locality_type = d.pop("localityType", UNSET)
        locality_type: BrowseResultItemLocalityType | Unset
        if isinstance(_locality_type, Unset):
            locality_type = UNSET
        else:
            locality_type = BrowseResultItemLocalityType(_locality_type)

        _administrative_area_type = d.pop("administrativeAreaType", UNSET)
        administrative_area_type: BrowseResultItemAdministrativeAreaType | Unset
        if isinstance(_administrative_area_type, Unset):
            administrative_area_type = UNSET
        else:
            administrative_area_type = BrowseResultItemAdministrativeAreaType(
                _administrative_area_type
            )

        _position = d.pop("position", UNSET)
        position: DisplayResponseCoordinate | Unset
        if isinstance(_position, Unset):
            position = UNSET
        else:
            position = DisplayResponseCoordinate.from_dict(_position)

        _access = d.pop("access", UNSET)
        access: list[ExtendedAccessPoint] | Unset = UNSET
        if _access is not UNSET:
            access = []
            for access_item_data in _access:
                access_item = ExtendedAccessPoint.from_dict(access_item_data)

                access.append(access_item)

        drive_through = d.pop("driveThrough", UNSET)

        distance = d.pop("distance", UNSET)

        excursion_distance = d.pop("excursionDistance", UNSET)

        _map_view = d.pop("mapView", UNSET)
        map_view: MapView | Unset
        if isinstance(_map_view, Unset):
            map_view = UNSET
        else:
            map_view = MapView.from_dict(_map_view)

        _categories = d.pop("categories", UNSET)
        categories: list[Category] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = Category.from_dict(categories_item_data)

                categories.append(categories_item)

        _chains = d.pop("chains", UNSET)
        chains: list[Chain] | Unset = UNSET
        if _chains is not UNSET:
            chains = []
            for chains_item_data in _chains:
                chains_item = Chain.from_dict(chains_item_data)

                chains.append(chains_item)

        _references = d.pop("references", UNSET)
        references: list[SupplierReference] | Unset = UNSET
        if _references is not UNSET:
            references = []
            for references_item_data in _references:
                references_item = SupplierReference.from_dict(references_item_data)

                references.append(references_item)

        _food_types = d.pop("foodTypes", UNSET)
        food_types: list[Category] | Unset = UNSET
        if _food_types is not UNSET:
            food_types = []
            for food_types_item_data in _food_types:
                food_types_item = Category.from_dict(food_types_item_data)

                food_types.append(food_types_item)

        _contacts = d.pop("contacts", UNSET)
        contacts: list[ContactInformation] | Unset = UNSET
        if _contacts is not UNSET:
            contacts = []
            for contacts_item_data in _contacts:
                contacts_item = ContactInformation.from_dict(contacts_item_data)

                contacts.append(contacts_item)

        _opening_hours = d.pop("openingHours", UNSET)
        opening_hours: list[OpeningHours] | Unset = UNSET
        if _opening_hours is not UNSET:
            opening_hours = []
            for opening_hours_item_data in _opening_hours:
                opening_hours_item = OpeningHours.from_dict(opening_hours_item_data)

                opening_hours.append(opening_hours_item)

        _time_zone = d.pop("timeZone", UNSET)
        time_zone: TimeZoneInfo | Unset
        if isinstance(_time_zone, Unset):
            time_zone = UNSET
        else:
            time_zone = TimeZoneInfo.from_dict(_time_zone)

        _extended = d.pop("extended", UNSET)
        extended: ExtendedAttribute | Unset
        if isinstance(_extended, Unset):
            extended = UNSET
        else:
            extended = ExtendedAttribute.from_dict(_extended)

        _phonemes = d.pop("phonemes", UNSET)
        phonemes: PhonemesSection | Unset
        if isinstance(_phonemes, Unset):
            phonemes = UNSET
        else:
            phonemes = PhonemesSection.from_dict(_phonemes)

        _media = d.pop("media", UNSET)
        media: Media | Unset
        if isinstance(_media, Unset):
            media = UNSET
        else:
            media = Media.from_dict(_media)

        _street_info = d.pop("streetInfo", UNSET)
        street_info: list[StreetInfo] | Unset = UNSET
        if _street_info is not UNSET:
            street_info = []
            for street_info_item_data in _street_info:
                street_info_item = StreetInfo.from_dict(street_info_item_data)

                street_info.append(street_info_item)

        _access_restrictions = d.pop("accessRestrictions", UNSET)
        access_restrictions: list[AccessRestrictionAttributes] | Unset = UNSET
        if _access_restrictions is not UNSET:
            access_restrictions = []
            for access_restrictions_item_data in _access_restrictions:
                access_restrictions_item = AccessRestrictionAttributes.from_dict(
                    access_restrictions_item_data
                )

                access_restrictions.append(access_restrictions_item)

        browse_result_item = cls(
            title=title,
            id=id,
            address=address,
            political_view=political_view,
            result_type=result_type,
            address_block_type=address_block_type,
            locality_type=locality_type,
            administrative_area_type=administrative_area_type,
            position=position,
            access=access,
            drive_through=drive_through,
            distance=distance,
            excursion_distance=excursion_distance,
            map_view=map_view,
            categories=categories,
            chains=chains,
            references=references,
            food_types=food_types,
            contacts=contacts,
            opening_hours=opening_hours,
            time_zone=time_zone,
            extended=extended,
            phonemes=phonemes,
            media=media,
            street_info=street_info,
            access_restrictions=access_restrictions,
        )

        browse_result_item.additional_properties = d
        return browse_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="TransitSpan")


@_attrs_define
class TransitSpan:
    r"""Contains information attached to a contiguous part of a `Section`. The information may be
    attached along different dimensions of a section which are geometry (spatial), distance or
    time.

    A section, if it uses spans, has an optional attribute `spans` which is an array of
    extended `Span` types.

    The attributes of a span which should be returned in the response are
    configured by a request parameter.

    Use this type as a base for any span extension for sections that provide spans.

        Attributes:
            offset (int | Unset): Offset of a coordinate in the section's polyline.
            length (int | Unset): Distance in meters. Example: 189.
            duration (int | Unset): Duration in seconds. Example: 198.
            country_code (str | Unset): ISO-3166-1 alpha-3 code Example: FRA.
            state_code (str | Unset): The second part of an ISO-3166-2 code (e.g., `TX` from `USA-TX`) consists of up to
                three alphanumeric characters.
                It is used to identify the principal subdivisions (e.g., provinces or states) of a country in conjunction with a
                CountryCode

                Note: State codes may not be available in some countries.
            names (list[LocalizedString] | Unset): Designated name for the span (e.g. a street name or a transport name)
            segment_id (str | Unset): **NOTE:** Attribute segmentId is deprecated. Use segmentRef instead.

                The directed topology segment id including prefix (e.g '+here:cm:segment:').

                The id consists of two parts.
                * The direction ('+' or '-')
                * followed by the topology segment id (a unique identifier within the HERE platform catalogs).

                The direction specifies whether the route is using the segment in its canonical direction ('+' aka traveling
                along the geometry's direction), or against it ('-' aka traveling against the geometry's direction).

                This attribute will not appear for HERE Public Transit v8 and HERE Intermodal Routing v8 requests
            segment_ref (str | Unset): A reference to the HMC topology segment used in this span.

                The standard representation of a segment reference has the following structure:
                {catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{segmentId}(#{direction}({startOffset}..{endOffset})?)?

                The individual parts are:
                * catalogHrn: The HERE Resource Name that identifies the source catalog of the segment, example:
                hrn:here:data::olp-here:rib-2
                * catalogVersion: The catalog version
                * layerId (optional): The layer inside the catalog where the segment can be found, example: topology-geometry
                * tileId: The HERE tile key of the partition/tile where the segment is located in the given version of the
                catalog. This can be on a lower level than the actual segment is stored at (for example, the provided tile ID
                can be on level 14, despite topology-geometry partitions being tiled at level 12). The level of a HERE tile key
                is indicated by the position of the highest set bit in binary representation. Since the HERE tile key represents
                a morton code of the x and y portion of the Tile ID, the level 12 tile ID can be retrieved from the level 14
                tile ID by removing the 4 least significant bits (or 2 bits per level) or 1 hexadecimal digit. For example, the
                level 14 tile 377894441 is included in the level 12 tile 23618402 (377894441<sub>10</sub> =
                16863629<sub>16</sub> &rightarrow; 1686362<sub>16</sub> = 23618402<sub>10</sub>)
                * segmentId: The identifier of the referenced topology segment inside the catalog, example:
                here:cm:segment:84905195
                * direction (optional): Either '*' for undirected or bidirectional, '+' for positive direction, '-' for negative
                direction, or '?' for unknown direction (not used by the routing service)
                * startOffset/endOffset (optional): The start- and end offset are non-negative numbers between 0 and 1,
                representing the start and end of the referenced range using a proportion of the length of the segment. 0
                represents the start and 1 the end of the segment, relative to the indicated direction (or positive direction in
                case of undirected segments). Example: 0.7..1

                Example of a segment reference in standard representation:
                hrn:here:data::olp-here:rib-2:1363::377894441:here:cm:segment:84905195#+0.7..1

                The segment references can also be provided in a compact representation, to reduce the response size. In the
                compact representation, some parts are replaced by placeholders, which can be resolved using the refReplacements
                dictionary in the parent section.
                The placeholder format is ```\$\d+``` and needs to be surrounded by colons or string start/end. It can be
                captured with the following regular expression: ```(^|:)\$\d+(:|$)/``` .

                Example of the segment reference previously mentioned in compact representation:
                $0:377894441:$1:84905195#+0.7..1
                With the corresponding refReplacements:
                "refReplacements": {
                  "0": "hrn:here:data::olp-here:rib-2:1363:",
                  "1": "here:cm:segment"
                }
            notices (list[int] | Unset): A list of indexes into the notices array of the parent section.
                References all notices that apply to the span.
    """

    offset: int | Unset = UNSET
    length: int | Unset = UNSET
    duration: int | Unset = UNSET
    country_code: str | Unset = UNSET
    state_code: str | Unset = UNSET
    names: list[LocalizedString] | Unset = UNSET
    segment_id: str | Unset = UNSET
    segment_ref: str | Unset = UNSET
    notices: list[int] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        offset = self.offset

        length = self.length

        duration = self.duration

        country_code = self.country_code

        state_code = self.state_code

        names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.names, Unset):
            names = []
            for names_item_data in self.names:
                names_item = names_item_data.to_dict()
                names.append(names_item)

        segment_id = self.segment_id

        segment_ref = self.segment_ref

        notices: list[int] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = self.notices

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if offset is not UNSET:
            field_dict["offset"] = offset
        if length is not UNSET:
            field_dict["length"] = length
        if duration is not UNSET:
            field_dict["duration"] = duration
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if state_code is not UNSET:
            field_dict["stateCode"] = state_code
        if names is not UNSET:
            field_dict["names"] = names
        if segment_id is not UNSET:
            field_dict["segmentId"] = segment_id
        if segment_ref is not UNSET:
            field_dict["segmentRef"] = segment_ref
        if notices is not UNSET:
            field_dict["notices"] = notices

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        offset = d.pop("offset", UNSET)

        length = d.pop("length", UNSET)

        duration = d.pop("duration", UNSET)

        country_code = d.pop("countryCode", UNSET)

        state_code = d.pop("stateCode", UNSET)

        _names = d.pop("names", UNSET)
        names: list[LocalizedString] | Unset = UNSET
        if _names is not UNSET:
            names = []
            for names_item_data in _names:
                names_item = LocalizedString.from_dict(names_item_data)

                names.append(names_item)

        segment_id = d.pop("segmentId", UNSET)

        segment_ref = d.pop("segmentRef", UNSET)

        notices = cast(list[int], d.pop("notices", UNSET))

        transit_span = cls(
            offset=offset,
            length=length,
            duration=duration,
            country_code=country_code,
            state_code=state_code,
            names=names,
            segment_id=segment_id,
            segment_ref=segment_ref,
            notices=notices,
        )

        transit_span.additional_properties = d
        return transit_span

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Polygon")


@_attrs_define
class Polygon:
    """A polygon is described by a boundary outer ring and a possible set of inner rings or holes.

    Attributes:
        outer (str | Unset): A special case of a Polyline where the first and last elements in the coordinates array are
            equivalent. Encoded as a string in [Flexible Polyline](https://github.com/heremaps/flexible-polyline) format.
            Coordinates are in the WGS84 coordinate system, including `Elevation` (if present). Example:
            BGk0owkBggpgrE1K6algB4KlgB3K3KlgB4KlgBmgB1KmgBqF2KkQA6a.
        inner (list[str] | Unset): A list of inner rings or holes for the polygon in the form of a list of LinearRings.
    """

    outer: str | Unset = UNSET
    inner: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        outer = self.outer

        inner: list[str] | Unset = UNSET
        if not isinstance(self.inner, Unset):
            inner = self.inner

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if outer is not UNSET:
            field_dict["outer"] = outer
        if inner is not UNSET:
            field_dict["inner"] = inner

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        outer = d.pop("outer", UNSET)

        inner = cast(list[str], d.pop("inner", UNSET))

        polygon = cls(
            outer=outer,
            inner=inner,
        )

        polygon.additional_properties = d
        return polygon

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ExtendedAccessPointType(str, Enum):
    DELIVERY = "delivery"
    EMERGENCY = "emergency"
    ENTRANCE = "entrance"
    LOADING = "loading"
    OTHER = "other"
    PARKING = "parking"
    TAXI = "taxi"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DynamicSpeedInfo")


@_attrs_define
class DynamicSpeedInfo:
    """Describes dynamic speed information, such as traffic speed, estimated speed without traffic, and turn time.

    Attributes:
        traffic_speed (float): Speed in meters per second
        base_speed (float): Speed in meters per second
        turn_time (int): Duration in seconds. Example: 198.
    """

    traffic_speed: float
    base_speed: float
    turn_time: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        traffic_speed = self.traffic_speed

        base_speed = self.base_speed

        turn_time = self.turn_time

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "trafficSpeed": traffic_speed,
            "baseSpeed": base_speed,
            "turnTime": turn_time,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        traffic_speed = d.pop("trafficSpeed")

        base_speed = d.pop("baseSpeed")

        turn_time = d.pop("turnTime")

        dynamic_speed_info = cls(
            traffic_speed=traffic_speed,
            base_speed=base_speed,
            turn_time=turn_time,
        )

        dynamic_speed_info.additional_properties = d
        return dynamic_speed_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class RelatedAddressRelationship(str, Enum):
    INTERSECTION = "intersection"
    MPA = "MPA"
    NEARBYADDRESS = "nearbyAddress"
    PARENTPA = "parentPA"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class EvPaymentSupportId(str, Enum):
    AUTHBYCARPLUGANDCHARGE = "authByCarPlugAndCharge"
    CREDITCARD = "creditCard"
    DEBITCARD = "debitCard"
    ONLINEAPPLEPAY = "onlineApplePay"
    ONLINEGOOGLEPAY = "onlineGooglePay"
    ONLINEPAYPAL = "onlinePaypal"
    OPERATORAPP = "operatorApp"

    def __str__(self) -> str:
        return str(self.value)



import argparse

import pandas as pd

# noinspection PyProtectedMember
from casageo.tools import CasaGeoClient, CasaGeoError, _consts, _util

# noinspection PyProtectedMember
from casageo.tools._spatial import CasaGeoSpatial, IsolinesResult, RoutesResult

__all__ = [
    "CasaGeoSpatial",
    "IsolinesResult",
    "RoutesResult",
]


def _main(args: argparse.Namespace) -> dict | pd.DataFrame:
    cga = CasaGeoClient(args.key, preferred_language=args.lang)
    cgs = CasaGeoSpatial(cga)
    match args.command:
        case "isolines":
            cgs.transport_mode = args.transport_mode
            cgs.routing_mode = args.routing_mode
            isolines_result = cgs.isolines(
                _util.lnglat_to_point(args.point),
                [float(x) for x in args.range.split(",")],
                range_type=args.rangetype,
            )
            if args.raw:
                return isolines_result.json()
            else:
                return isolines_result.dataframe()
        case "routes":
            cgs.routing_mode = args.routing_mode
            cgs.transport_mode = args.transport_mode
            routes_result = cgs.routes(
                _util.lnglat_to_point(args.pointa),
                _util.lnglat_to_point(args.pointb),
            )
            if args.raw:
                return routes_result.json()
            else:
                return routes_result.dataframe()
        case "get-info":
            return cga.account_info().json()
        case _:
            raise CasaGeoError(f"Unknown command: {args.command}")


if __name__ == "__main__":
    _util.cli_main(_consts.get_spatial_parser(), _main)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.road_info_type import RoadInfoType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_route_number import LocalizedRouteNumber
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="RoadInfo")


@_attrs_define
class RoadInfo:
    """Road information attached to an offset action

    Example:
        {'fennstrasse': {'type': 'street', 'name': [{'value': 'Fennstraße', 'language': 'de'}], 'number': [{'value':
            'B96', 'language': 'de'}], 'toward': [{'value': 'Reinickendorf', 'language': 'de'}]}}

    Attributes:
        type_ (RoadInfoType | Unset): Type of the road (rural, urban, highway)
        name (list[LocalizedString] | Unset): Name of the road

            If the road has multiple names, each name will be a separate entry in the array.
            The road names can be in multiple languages. If a preferred language was provided,
            and names in that language are available, they will be prioritized in the array.
            Otherwise the default name of the street is prioritized.
        number (list[LocalizedRouteNumber] | Unset): Route name or number (e.g. 'M25')
        toward (list[LocalizedString] | Unset): Names of destinations on sign which can be reached when going in that
            direction
    """

    type_: RoadInfoType | Unset = UNSET
    name: list[LocalizedString] | Unset = UNSET
    number: list[LocalizedRouteNumber] | Unset = UNSET
    toward: list[LocalizedString] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        name: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.name, Unset):
            name = []
            for name_item_data in self.name:
                name_item = name_item_data.to_dict()
                name.append(name_item)

        number: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.number, Unset):
            number = []
            for number_item_data in self.number:
                number_item = number_item_data.to_dict()
                number.append(number_item)

        toward: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.toward, Unset):
            toward = []
            for toward_item_data in self.toward:
                toward_item = toward_item_data.to_dict()
                toward.append(toward_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_
        if name is not UNSET:
            field_dict["name"] = name
        if number is not UNSET:
            field_dict["number"] = number
        if toward is not UNSET:
            field_dict["toward"] = toward

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_route_number import LocalizedRouteNumber
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        _type_ = d.pop("type", UNSET)
        type_: RoadInfoType | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = RoadInfoType(_type_)

        _name = d.pop("name", UNSET)
        name: list[LocalizedString] | Unset = UNSET
        if _name is not UNSET:
            name = []
            for name_item_data in _name:
                name_item = LocalizedString.from_dict(name_item_data)

                name.append(name_item)

        _number = d.pop("number", UNSET)
        number: list[LocalizedRouteNumber] | Unset = UNSET
        if _number is not UNSET:
            number = []
            for number_item_data in _number:
                number_item = LocalizedRouteNumber.from_dict(number_item_data)

                number.append(number_item)

        _toward = d.pop("toward", UNSET)
        toward: list[LocalizedString] | Unset = UNSET
        if _toward is not UNSET:
            toward = []
            for toward_item_data in _toward:
                toward_item = LocalizedString.from_dict(toward_item_data)

                toward.append(toward_item)

        road_info = cls(
            type_=type_,
            name=name,
            number=number,
            toward=toward,
        )

        road_info.additional_properties = d
        return road_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PedestrianSummary")


@_attrs_define
class PedestrianSummary:
    """Total value of key attributes for a route section.

    Attributes:
        duration (int): Duration in seconds. Example: 198.
        length (int): Distance in meters. Example: 189.
        base_duration (int | Unset): Duration in seconds. Example: 198.
    """

    duration: int
    length: int
    base_duration: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        duration = self.duration

        length = self.length

        base_duration = self.base_duration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "duration": duration,
            "length": length,
        })
        if base_duration is not UNSET:
            field_dict["baseDuration"] = base_duration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        duration = d.pop("duration")

        length = d.pop("length")

        base_duration = d.pop("baseDuration", UNSET)

        pedestrian_summary = cls(
            duration=duration,
            length=length,
            base_duration=base_duration,
        )

        pedestrian_summary.additional_properties = d
        return pedestrian_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EvAvailabilityConnector")


@_attrs_define
class EvAvailabilityConnector:
    """
    Attributes:
        id (str | Unset): HERE ID of the connector.
        cpo_id (str | Unset): CPO ID of the connector.
        type_id (str | Unset): Connector type ID. For more information on the current connector types,
            see [resource-type-connector-types.html](https://www.here.com/docs/bundle/ev-charge-points-api-developer-
            guide/page/topics/resource-type-connector-types.html)
        max_power_level (float | Unset): Maximum charge power of connector in kilowatts.
    """

    id: str | Unset = UNSET
    cpo_id: str | Unset = UNSET
    type_id: str | Unset = UNSET
    max_power_level: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        cpo_id = self.cpo_id

        type_id = self.type_id

        max_power_level = self.max_power_level

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if cpo_id is not UNSET:
            field_dict["cpoId"] = cpo_id
        if type_id is not UNSET:
            field_dict["typeId"] = type_id
        if max_power_level is not UNSET:
            field_dict["maxPowerLevel"] = max_power_level

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        cpo_id = d.pop("cpoId", UNSET)

        type_id = d.pop("typeId", UNSET)

        max_power_level = d.pop("maxPowerLevel", UNSET)

        ev_availability_connector = cls(
            id=id,
            cpo_id=cpo_id,
            type_id=type_id,
            max_power_level=max_power_level,
        )

        ev_availability_connector.additional_properties = d
        return ev_availability_connector

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AutosuggestQueryResultItemResultType(str, Enum):
    CATEGORYQUERY = "categoryQuery"
    CHAINQUERY = "chainQuery"

    def __str__(self) -> str:
        return str(self.value)



"""Contains all the data models used in inputs/outputs"""

from .schema_retrieve_format import SchemaRetrieveFormat
from .schema_retrieve_lang import SchemaRetrieveLang
from .schema_retrieve_response_200 import SchemaRetrieveResponse200
from .v1_isolines_create_optimize_for import V1IsolinesCreateOptimizeFor
from .v1_isolines_create_rangetype import V1IsolinesCreateRangetype
from .v1_isolines_create_routing_mode import V1IsolinesCreateRoutingMode
from .v1_isolines_create_transport_mode import V1IsolinesCreateTransportMode
from .v1_routes_create_routing_mode import V1RoutesCreateRoutingMode
from .v1_routes_create_transport_mode import V1RoutesCreateTransportMode
from .v1_schema_retrieve_format import V1SchemaRetrieveFormat
from .v1_schema_retrieve_lang import V1SchemaRetrieveLang
from .v1_schema_retrieve_response_200 import V1SchemaRetrieveResponse200

__all__ = (
    "SchemaRetrieveFormat",
    "SchemaRetrieveLang",
    "SchemaRetrieveResponse200",
    "V1IsolinesCreateOptimizeFor",
    "V1IsolinesCreateRangetype",
    "V1IsolinesCreateRoutingMode",
    "V1IsolinesCreateTransportMode",
    "V1RoutesCreateRoutingMode",
    "V1RoutesCreateTransportMode",
    "V1SchemaRetrieveFormat",
    "V1SchemaRetrieveLang",
    "V1SchemaRetrieveResponse200",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CountryInfo")


@_attrs_define
class CountryInfo:
    """
    Attributes:
        alpha2 (str | Unset): [ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) country code
        alpha3 (str | Unset): [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country code
    """

    alpha2: str | Unset = UNSET
    alpha3: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        alpha2 = self.alpha2

        alpha3 = self.alpha3

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alpha2 is not UNSET:
            field_dict["alpha2"] = alpha2
        if alpha3 is not UNSET:
            field_dict["alpha3"] = alpha3

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        alpha2 = d.pop("alpha2", UNSET)

        alpha3 = d.pop("alpha3", UNSET)

        country_info = cls(
            alpha2=alpha2,
            alpha3=alpha3,
        )

        country_info.additional_properties = d
        return country_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_place import BasePlace


T = TypeVar("T", bound="Departure")


@_attrs_define
class Departure:
    """Describe a departure or arrival location and time.

    Attributes:
        place (BasePlace):  Example: {'type': 'place', 'location': {'lat': 50.339167, 'lng': 18.93}}.
        time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset).
    """

    place: BasePlace
    time: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        place = self.place.to_dict()

        time: str | Unset = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "place": place,
        })
        if time is not UNSET:
            field_dict["time"] = time

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.base_place import BasePlace

        d = dict(src_dict)
        place = BasePlace.from_dict(d.pop("place"))

        _time = d.pop("time", UNSET)
        time: datetime.datetime | Unset
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = isoparse(_time)

        departure = cls(
            place=place,
            time=time,
        )

        departure.additional_properties = d
        return departure

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import functools
import json
import logging
import os
import unittest
from contextlib import contextmanager
from datetime import datetime
from unittest.mock import patch

import httpx
from shapely import Point

from casageo.coder import CasaGeoCoder
from casageo.spatial import CasaGeoSpatial
from casageo.tools import CasaGeoClient, CasaGeoError

API_KEY = "XXXXX-XXXXX-XXXXX-XXXXX"
FIXTURE_DIR = os.path.join(os.path.dirname(__file__), "fixtures")

POINT_IZET = Point(9.4854461, 53.9580118)
POINT_HH = Point(10.008223, 53.553089)

casageo_logger = logging.getLogger("casageo.tools")
casageo_logger.setLevel(logging.ERROR)  # Ignore warnings about caught errors.


@functools.cache
def load_fixture(name: str):
    with open(os.path.join(FIXTURE_DIR, f"{name}.json"), mode="rb") as f:
        return json.load(f)


@contextmanager
def mock_request(apiclient, fixture):
    # TODO: Remove the need for the apiclient parameter by unifying the API clients.
    with patch("httpx.Client.request", autospec=True) as mock:
        mock.return_value = httpx.Response(**fixture["response"])
        yield mock
        mock.assert_called_once_with(apiclient.get_httpx_client(), **fixture["request"])


class TestAccountInfo(unittest.TestCase):
    def setUp(self):
        self.client = CasaGeoClient(API_KEY)
        self.client.preferred_language = "de,en"

    def test_account_info_success(self):
        with mock_request(self.client._v1_client, load_fixture("account_info_1")):
            result = self.client.account_info()
        self.assertTrue(result)
        self.assertEqual("testuser", result.username())
        self.assertEqual("Professional", result.account_type())
        self.assertEqual(43359, result.credits())
        self.assertEqual(
            datetime.fromisoformat("2026-11-18T15:45:09Z"), result.expires()
        )

    def test_account_info_failure(self):
        with patch("httpx.Client.request", return_value=httpx.Response(500)):
            result = self.client.account_info()

        self.assertFalse(result)
        self.assertIsInstance(result.error(), CasaGeoError)
        self.assertEqual("Bad status code 500: b''", str(result.error()))


class TestCoder(unittest.TestCase):
    def setUp(self):
        self.client = CasaGeoClient(API_KEY)
        self.client.preferred_language = "de,en"

        self.cgc = CasaGeoCoder(self.client)

    def test_address(self):
        with mock_request(
            self.client._geocoding_client, load_fixture("coder_address_1")
        ):
            result = self.cgc.address("Fraunhoferstr. 3, 25524 Itzehoe DEU")

        self.assertTrue(result)
        # TODO: More assertions.

        df = result.dataframe()
        self.assertEqual(1, len(df))
        self.assertEqual(
            "Fraunhoferstraße 3, 25524 Itzehoe, Deutschland", df["address"][0]
        )
        # TODO: More assertions.

        # TODO: Test the results of dataframe() parameters.


class TestSpatialClient(unittest.TestCase):
    def setUp(self):
        self.client = CasaGeoClient(API_KEY)
        self.client.preferred_language = "de,en"

        self.cgs = CasaGeoSpatial(self.client)
        self.cgs.transport_mode = "car"
        self.cgs.routing_mode = "fast"

    def test_isolines(self):
        with mock_request(
            self.client._isolines_client, load_fixture("spatial_isolines_1")
        ):
            result = self.cgs.isolines(POINT_HH, [3, 9, 15], range_type="time")

        self.assertTrue(result)
        # TODO: More assertions.

        df = result.dataframe(departure_info=True)
        self.assertEqual(3, len(df))
        # TODO: More assertions.

    def test_routes(self):
        with mock_request(
            self.client._routing_client, load_fixture("spatial_routes_1")
        ):
            result = self.cgs.routes(POINT_IZET, POINT_HH)

        self.assertTrue(result)
        # TODO: More assertions.

        df = result.dataframe(departure_info=True, arrival_info=True)
        self.assertEqual(1, len(df))
        # TODO: More assertions.


if __name__ == "__main__":
    unittest.main()



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.allow import Allow
from ...models.auth_error_response_schema import AuthErrorResponseSchema
from ...models.avoid import Avoid
from ...models.ev_empirical_model import EVEmpiricalModel
from ...models.ev_physical_model import EVPhysicalModel
from ...models.exclude import Exclude
from ...models.fuel import Fuel
from ...models.match_trace import MatchTrace
from ...models.return_ import Return
from ...models.router_mode import RouterMode
from ...models.router_route_response import RouterRouteResponse
from ...models.routing_error_response import RoutingErrorResponse
from ...models.scooter import Scooter
from ...models.spans import Spans
from ...models.taxi import Taxi
from ...models.truck import Truck
from ...models.units import Units
from ...models.vehicle import Vehicle
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: MatchTrace | Unset = UNSET,
    transport_mode: RouterMode,
    departure_time: str | Unset = UNSET,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    json_transport_mode = transport_mode.value
    params["transportMode"] = json_transport_mode

    params["departureTime"] = departure_time

    json_avoid: dict[str, Any] | Unset = UNSET
    if not isinstance(avoid, Unset):
        json_avoid = avoid.to_dict()
    if not isinstance(json_avoid, Unset):
        params.update((f"avoid[{k}]", v) for k, v in json_avoid.items())

    json_allow: dict[str, Any] | Unset = UNSET
    if not isinstance(allow, Unset):
        json_allow = allow.to_dict()
    if not isinstance(json_allow, Unset):
        params.update((f"allow[{k}]", v) for k, v in json_allow.items())

    json_exclude: dict[str, Any] | Unset = UNSET
    if not isinstance(exclude, Unset):
        json_exclude = exclude.to_dict()
    if not isinstance(json_exclude, Unset):
        params.update((f"exclude[{k}]", v) for k, v in json_exclude.items())

    json_units: str | Unset = UNSET
    if not isinstance(units, Unset):
        json_units = units.value

    params["units"] = json_units

    json_lang: list[str] | Unset = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang

    params["lang"] = json_lang

    json_return_: list[str] | Unset = UNSET
    if not isinstance(return_, Unset):
        json_return_ = []
        for return_item_data in return_:
            return_item = return_item_data.value
            json_return_.append(return_item)

    params["return"] = json_return_

    json_spans: list[str] | Unset = UNSET
    if not isinstance(spans, Unset):
        json_spans = []
        for spans_item_data in spans:
            spans_item = spans_item_data.value
            json_spans.append(spans_item)

    params["spans"] = json_spans

    json_truck: dict[str, Any] | Unset = UNSET
    if not isinstance(truck, Unset):
        json_truck = truck.to_dict()
    if not isinstance(json_truck, Unset):
        params.update((f"truck[{k}]", v) for k, v in json_truck.items())

    json_vehicle: dict[str, Any] | Unset = UNSET
    if not isinstance(vehicle, Unset):
        json_vehicle = vehicle.to_dict()
    if not isinstance(json_vehicle, Unset):
        params.update((f"vehicle[{k}]", v) for k, v in json_vehicle.items())

    params["consumptionModel"] = consumption_model

    json_ev: dict[str, Any] | Unset
    if isinstance(ev, Unset):
        json_ev = UNSET
    elif isinstance(ev, EVEmpiricalModel):
        json_ev = ev.to_dict()
    else:
        json_ev = ev.to_dict()

    params["ev"] = json_ev

    json_fuel: dict[str, Any] | Unset = UNSET
    if not isinstance(fuel, Unset):
        json_fuel = fuel.to_dict()
    if not isinstance(json_fuel, Unset):
        params.update((f"fuel[{k}]", v) for k, v in json_fuel.items())

    json_scooter: dict[str, Any] | Unset = UNSET
    if not isinstance(scooter, Unset):
        json_scooter = scooter.to_dict()
    if not isinstance(json_scooter, Unset):
        params.update((f"scooter[{k}]", v) for k, v in json_scooter.items())

    json_taxi: dict[str, Any] | Unset = UNSET
    if not isinstance(taxi, Unset):
        json_taxi = taxi.to_dict()
    if not isinstance(json_taxi, Unset):
        params.update((f"taxi[{k}]", v) for k, v in json_taxi.items())

    params["billingTag"] = billing_tag

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/import",
        "params": params,
    }

    if not isinstance(body, Unset):
        _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    if response.status_code == 200:
        response_200 = RouterRouteResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = RoutingErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = AuthErrorResponseSchema.from_dict(response.json())

        return response_401

    if response.status_code == 403:

        def _parse_response_403(
            data: object,
        ) -> AuthErrorResponseSchema | RoutingErrorResponse:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_routing_403_error_response_schema_type_0 = (
                    AuthErrorResponseSchema.from_dict(data)
                )

                return componentsschemas_routing_403_error_response_schema_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_routing_403_error_response_schema_type_1 = (
                RoutingErrorResponse.from_dict(data)
            )

            return componentsschemas_routing_403_error_response_schema_type_1

        response_403 = _parse_response_403(response.json())

        return response_403

    if response.status_code == 500:
        response_500 = RoutingErrorResponse.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: MatchTrace | Unset = UNSET,
    transport_mode: RouterMode,
    departure_time: str | Unset = UNSET,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    """Calculate a route from a sequence of trace points

     Creates a route from a sequence of trace points.

    Post body size limit is 10MiB.

    For best results, use 1Hz GPS data or any points that have a spacing of a few meters between them.
    For traces with less frequent points, the Route Import service will attempt to create an approximate
    reconstruction.
    In some situations, when consecutive points are too far apart (more than about 30 kilometers of on-
    road distance), they could be considered unreachable and one of them could fail to be matched.

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        scooter (Scooter | Unset): Scooter-specific parameters
        taxi (Taxi | Unset): Taxi-specific parameters
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (MatchTrace | Unset): Trace file with points and path match parameters Example:
            {'trace': [{'lat': 52.0, 'lng': 13.1}, {'lat': 52.1, 'lng': 13.2}, {'lat': 52.2, 'lng':
            13.3}], 'via': [{'index': 1}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        transport_mode=transport_mode,
        departure_time=departure_time,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        scooter=scooter,
        taxi=taxi,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    body: MatchTrace | Unset = UNSET,
    transport_mode: RouterMode,
    departure_time: str | Unset = UNSET,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    """Calculate a route from a sequence of trace points

     Creates a route from a sequence of trace points.

    Post body size limit is 10MiB.

    For best results, use 1Hz GPS data or any points that have a spacing of a few meters between them.
    For traces with less frequent points, the Route Import service will attempt to create an approximate
    reconstruction.
    In some situations, when consecutive points are too far apart (more than about 30 kilometers of on-
    road distance), they could be considered unreachable and one of them could fail to be matched.

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        scooter (Scooter | Unset): Scooter-specific parameters
        taxi (Taxi | Unset): Taxi-specific parameters
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (MatchTrace | Unset): Trace file with points and path match parameters Example:
            {'trace': [{'lat': 52.0, 'lng': 13.1}, {'lat': 52.1, 'lng': 13.2}, {'lat': 52.2, 'lng':
            13.3}], 'via': [{'index': 1}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse
    """

    return sync_detailed(
        client=client,
        body=body,
        transport_mode=transport_mode,
        departure_time=departure_time,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        scooter=scooter,
        taxi=taxi,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: MatchTrace | Unset = UNSET,
    transport_mode: RouterMode,
    departure_time: str | Unset = UNSET,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
]:
    """Calculate a route from a sequence of trace points

     Creates a route from a sequence of trace points.

    Post body size limit is 10MiB.

    For best results, use 1Hz GPS data or any points that have a spacing of a few meters between them.
    For traces with less frequent points, the Route Import service will attempt to create an approximate
    reconstruction.
    In some situations, when consecutive points are too far apart (more than about 30 kilometers of on-
    road distance), they could be considered unreachable and one of them could fail to be matched.

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        scooter (Scooter | Unset): Scooter-specific parameters
        taxi (Taxi | Unset): Taxi-specific parameters
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (MatchTrace | Unset): Trace file with points and path match parameters Example:
            {'trace': [{'lat': 52.0, 'lng': 13.1}, {'lat': 52.1, 'lng': 13.2}, {'lat': 52.2, 'lng':
            13.3}], 'via': [{'index': 1}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        transport_mode=transport_mode,
        departure_time=departure_time,
        avoid=avoid,
        allow=allow,
        exclude=exclude,
        units=units,
        lang=lang,
        return_=return_,
        spans=spans,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        scooter=scooter,
        taxi=taxi,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: MatchTrace | Unset = UNSET,
    transport_mode: RouterMode,
    departure_time: str | Unset = UNSET,
    avoid: Avoid | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    units: Units | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    return_: list[Return] | Unset = UNSET,
    spans: list[Spans] | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: EVEmpiricalModel | EVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    scooter: Scooter | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | RoutingErrorResponse
    | RouterRouteResponse
    | RoutingErrorResponse
    | None
):
    """Calculate a route from a sequence of trace points

     Creates a route from a sequence of trace points.

    Post body size limit is 10MiB.

    For best results, use 1Hz GPS data or any points that have a spacing of a few meters between them.
    For traces with less frequent points, the Route Import service will attempt to create an approximate
    reconstruction.
    In some situations, when consecutive points are too far apart (more than about 30 kilometers of on-
    road distance), they could be considered unreachable and one of them could fail to be matched.

    Args:
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        departure_time (str | Unset): Specifies the time either as

            * "**RFC 3339**, section 5.6 as defined by either `date-time` or `date-only` 'T'
            `partial-time` (ie no time-offset)", or
            * the special value `any` which stand for unspecified time
        avoid (Avoid | Unset):
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        units (Units | Unset): Units of measurement used, for example, in guidance instructions.
            The default is `metric`.
        lang (list[str] | Unset):
        return_ (list[Return] | Unset):
        spans (list[Spans] | Unset):
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (EVEmpiricalModel | EVPhysicalModel | Unset): EV properties to be used for calculating
            consumption.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`

            The following `ev` properties are additionally required in order to calculate State-of-
            Charge along the route:
            * `initialCharge`
            * `maxCharge`

            Route reachability based on State-of-Charge will be evaluated for the following
            constraints, if additionally provided,
            * `minChargeAtDestination`
            * `minChargeAtFirstChargingStation`
            * `minChargeAtChargingStation`

            The following properties are additionally required in order to calculate charging at
            charging station waypoints (see documentation for `via` parameter)
            * `chargingCurve`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * EV parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        fuel (Fuel | Unset): **Disclaimer: This parameter is currently in beta release, and is
            therefore subject to breaking changes.**

            Fuel parameters to be used for calculating consumption and related CO2 emission, and toll
            calculation.

            The following attributes are required for calculating consumption:
              * `type`
              * `freeFlowSpeedTable`

            The following attribute is needed for fuel specific toll calculation (if not provided
            default toll instead of fuel specific toll will be returned):
              * `type`

            **Notes**:
            * Hybrid vehicles (EV + Other fuel types) are not supported. Consumption properties are
            not supported for combination of `ev` and `fuel` vehicles.
            * Fuel parameters are not supported in combination with pedestrian and bicycle
            `transportMode`.
        scooter (Scooter | Unset): Scooter-specific parameters
        taxi (Taxi | Unset): Taxi-specific parameters
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (MatchTrace | Unset): Trace file with points and path match parameters Example:
            {'trace': [{'lat': 52.0, 'lng': 13.1}, {'lat': 52.1, 'lng': 13.2}, {'lat': 52.2, 'lng':
            13.3}], 'via': [{'index': 1}]}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | RoutingErrorResponse | RouterRouteResponse | RoutingErrorResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            transport_mode=transport_mode,
            departure_time=departure_time,
            avoid=avoid,
            allow=allow,
            exclude=exclude,
            units=units,
            lang=lang,
            return_=return_,
            spans=spans,
            truck=truck,
            vehicle=vehicle,
            consumption_model=consumption_model,
            ev=ev,
            fuel=fuel,
            scooter=scooter,
            taxi=taxi,
            billing_tag=billing_tag,
            x_request_id=x_request_id,
        )
    ).parsed
